@@ -32,10 +32,14 @@ in this Software without prior written authorization from The Open Group.
 #include <config.h>
 #endif
 #include "Xlibint.h"
+#if USE_XCB
+#include "Xxcbint.h"
+#else /* !USE_XCB */
 #include <X11/Xtrans/Xtrans.h>
+#include <X11/extensions/bigreqstr.h>
+#endif /* USE_XCB */
 #include <X11/Xatom.h>
 #include <X11/Xresource.h>
-#include <X11/extensions/bigreqstr.h>
 #include <stdio.h>
 #include "Xintconn.h"
 
@@ -43,6 +47,7 @@ in this Software without prior written authorization from The Open Group.
 #include "XKBlib.h"
 #endif /* XKB */
 
+#if !USE_XCB
 #ifdef X_NOT_POSIX
 #define Size_t unsigned int
 #else
@@ -55,6 +60,7 @@ typedef struct {
     unsigned long seq;
     int opcode;
 } _XBigReqState;
+#endif /* !USE_XCB */
 
 #ifdef XTHREADS
 #include "locking.h"
@@ -73,8 +79,10 @@ static xReq _dummy_request = {
 };
 
 static void OutOfMemory(Display *dpy, char *setup);
+#if !USE_XCB
 static Bool _XBigReqHandler(Display *dpy, xReply *rep, char *buf, int len,
 				XPointer data);
+#endif /* !USE_XCB */
 
 /* 
  * Connects to a server, creates a Display object and returns a pointer to
@@ -87,16 +95,20 @@ XOpenDisplay (
 	register Display *dpy;		/* New Display object being created. */
 	register int i;
 	int j, k;			/* random iterator indexes */
+#if !USE_XCB
 	char *display_name;		/* pointer to display name */
 	int endian;			/* to determine which endian. */
 	xConnClientPrefix client;	/* client information */
-	xConnSetupPrefix prefix;	/* prefix information */
-	int vendorlen;			/* length of vendor string */
+	int idisplay;			/* display number */
+	int prefixread = 0;             /* setup prefix already read? */
+	char *conn_auth_name, *conn_auth_data;
+	int conn_auth_namelen, conn_auth_datalen;
+#endif /* !USE_XCB */
 	char *setup = NULL;		/* memory allocated at startup */
 	char *fullname = NULL;		/* expanded name of display */
-	int idisplay;			/* display number */
 	int iscreen;			/* screen number */
-	int prefixread = 0;             /* setup prefix already read? */
+	xConnSetupPrefix prefix;	/* prefix information */
+	int vendorlen;			/* length of vendor string */
 	union {
 		xConnSetup *setup;
 		char *failure;
@@ -108,12 +120,11 @@ XOpenDisplay (
 	} u;				/* proto data returned from server */
 	long setuplength;	/* number of bytes in setup message */
 	long usedbytes = 0;     /* number of bytes we have processed */
-	char *conn_auth_name, *conn_auth_data;
-	int conn_auth_namelen, conn_auth_datalen;
 	unsigned long mask;
        long int conn_buf_size;
        char *xlib_buffer_size;
 
+#if !USE_XCB
 	bzero((char *) &client, sizeof(client));
 	bzero((char *) &prefix, sizeof(prefix));
 
@@ -131,6 +142,8 @@ XOpenDisplay (
 		/* Display is non-NULL, copy the pointer */
 		display_name = (char *)display;
 	}
+#endif /* !USE_XCB */
+
 /*
  * Set the default error handlers.  This allows the global variables to
  * default to NULL for use with shared libraries.
@@ -151,6 +164,12 @@ XOpenDisplay (
  * will set fullname to point to the expanded name.
  */
 
+#if USE_XCB
+	if(!_XConnectXCB(dpy, display, &fullname, &iscreen)) {
+		OutOfMemory(dpy, 0);
+		return 0;
+	}
+#else /* !USE_XCB */
 	if ((dpy->trans_conn = _X11TransConnectDisplay (
 					 display_name, &fullname, &idisplay,
 					 &iscreen, &conn_auth_name,
@@ -161,6 +180,7 @@ XOpenDisplay (
 	}
 
 	dpy->fd = _X11TransGetConnectionNumber (dpy->trans_conn);
+#endif /* USE_XCB */
 
 	/* Initialize as much of the display structure as we can.
 	 * Initialize pointers to NULL so that XFreeDisplayStructure will
@@ -240,6 +260,13 @@ XOpenDisplay (
 		return(NULL);
 	}
 
+#if USE_XCB
+	if (!_XCBInitDisplayLock(dpy)) {
+	        OutOfMemory (dpy, setup);
+		return(NULL);
+	}
+#endif
+
 	if (!_XPollfdCacheInit(dpy)) {
 	        OutOfMemory (dpy, setup);
 		return(NULL);
@@ -277,6 +304,7 @@ XOpenDisplay (
 	    return(NULL);
 	}
 
+#if !USE_XCB
 /*
  * The xConnClientPrefix describes the initial connection setup information
  * and is followed by the authorization information.  Sites that are interested
@@ -331,7 +359,18 @@ XOpenDisplay (
 	    Xfree ((char *)dpy);
 	    return(NULL);
 	}
+#endif /* !USE_XCB */
 
+#if USE_XCB
+	{
+		const struct xcb_setup_t *xcbsetup = xcb_get_setup(dpy->xcb->connection);
+		memcpy(&prefix, xcbsetup, sizeof(prefix));
+		setuplength = prefix.length << 2;
+		setup = (char *) xcbsetup;
+		setup += SIZEOF(xConnSetupPrefix);
+		u.setup = (xConnSetup *) setup;
+	}
+#else /* !USE_XCB */
 	setuplength = prefix.length << 2;
 	if ( (u.setup = (xConnSetup *)
 	      (setup =  Xmalloc ((unsigned) setuplength))) == NULL) {
@@ -362,6 +401,7 @@ XOpenDisplay (
 		OutOfMemory(dpy, setup);
 		return (NULL);
 	}
+#endif /* USE_XCB */
 
 /*
  * Check if the reply was long enough to get any information out of it.
@@ -438,11 +478,8 @@ XOpenDisplay (
   	(void) strncpy(dpy->vendor, u.vendor, vendorlen);
 	dpy->vendor[vendorlen] = '\0';
  	vendorlen = (vendorlen + 3) & ~3;	/* round up */
+	u.vendor += vendorlen;
 
-
-	memmove (setup, u.vendor + vendorlen,
-		 (int) setuplength - sz_xConnSetup - vendorlen);
- 	u.vendor = setup;
 /*
  * Now iterate down setup information.....
  */
@@ -595,7 +632,9 @@ XOpenDisplay (
  * Now start talking to the server to setup all other information...
  */
 
+#if !USE_XCB
 	Xfree (setup);	/* all finished with setup information */
+#endif /* !USE_XCB */
 
 /*
  * Make sure default screen is legal.
@@ -605,10 +644,12 @@ XOpenDisplay (
 	    return(NULL);
 	}
 
+#if !USE_XCB
 /*
  * finished calling internal routines, now unlock for external routines
  */
 	UnlockDisplay(dpy);
+#endif /* !USE_XCB */
 
 /*
  * Set up other stuff clients are always going to use.
@@ -631,17 +672,24 @@ XOpenDisplay (
  */
 	(void) XSynchronize(dpy, _Xdebug);
 
+#if USE_XCB
+	dpy->bigreq_size = xcb_get_maximum_request_length(dpy->xcb->connection);
+	if(dpy->bigreq_size <= dpy->max_request_size)
+		dpy->bigreq_size = 0;
+#endif /* USE_XCB */
+
 /*
  * get availability of large requests, and
  * get the resource manager database off the root window.
  */
 	LockDisplay(dpy);
 	{
+	    xGetPropertyReply reply;
+	    xGetPropertyReq *req;
+#if !USE_XCB
 	    _XAsyncHandler async;
 	    _XBigReqState async_state;
 	    xQueryExtensionReq *qreq;
-	    xGetPropertyReply reply;
-	    xGetPropertyReq *req;
 	    xBigReqEnableReq *breq;
 	    xBigReqEnableReply brep;
 
@@ -655,6 +703,7 @@ XOpenDisplay (
 	    qreq->nbytes = bignamelen;
 	    qreq->length += (bignamelen+3)>>2;
 	    Data(dpy, XBigReqExtensionName, bignamelen);
+#endif /* !USE_XCB */
 
 	    GetReq (GetProperty, req);
 	    req->window = RootWindow(dpy, 0);
@@ -675,6 +724,7 @@ XOpenDisplay (
 		else if (reply.propertyType != None)
 		    _XEatData(dpy, reply.nItems * (reply.format >> 3));
 	    }
+#if !USE_XCB
 	    DeqAsyncHandler(dpy, &async);
 	    if (async_state.opcode) {
 		GetReq(BigReqEnable, breq);
@@ -683,6 +733,7 @@ XOpenDisplay (
 		if (_XReply(dpy, (xReply *)&brep, 0, xFalse))
 		    dpy->bigreq_size = brep.max_request_size;
 	    }
+#endif /* !USE_XCB */
 	}
 	UnlockDisplay(dpy);
 
@@ -701,13 +752,14 @@ XOpenDisplay (
  	return(dpy);
 }
 
+#if !USE_XCB
 static Bool
-_XBigReqHandler(dpy, rep, buf, len, data)
-    register Display *dpy;
-    register xReply *rep;
-    char *buf;
-    int len;
-    XPointer data;
+_XBigReqHandler(
+    register Display *dpy,
+    register xReply *rep,
+    char *buf,
+    int len,
+    XPointer data)
 {
     _XBigReqState *state;
     xQueryExtensionReply replbuf;
@@ -726,6 +778,7 @@ _XBigReqHandler(dpy, rep, buf, len, data)
 	state->opcode = repl->major_opcode;
     return True;
 }
+#endif /* !USE_XCB */
 
 
 /* XFreeDisplayStructure frees all the storage associated with a 
@@ -853,6 +906,10 @@ void _XFreeDisplayStructure(dpy)
 	if (dpy->filedes)
 	    Xfree (dpy->filedes);
 
+#if USE_XCB
+	_XFreeX11XCBStructure(dpy);
+#endif /* USE_XCB */
+
 	Xfree ((char *)dpy);
 }
 
@@ -863,7 +920,14 @@ static void OutOfMemory (dpy, setup)
     Display *dpy;
     char *setup;
 {
+#if USE_XCB
+    if(dpy->xcb->connection)
+	xcb_disconnect(dpy->xcb->connection);
+#else /* !USE_XCB */
     _XDisconnectDisplay (dpy->trans_conn);
+#endif /* USE_XCB */
     _XFreeDisplayStructure (dpy);
+#if !USE_XCB
     if (setup) Xfree (setup);
+#endif /* !USE_XCB */
 }
